from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from plone.base import PloneMessageFactory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import implementer


@implementer(IStructureAction)
class TagsAction:

    template = ViewPageTemplateFile("templates/tags.pt")
    order = 6

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        base_vocabulary = "%s/@@getVocabulary?name=" % getSite().absolute_url()
        return {
            "tooltip": translate(_("Tags"), context=self.request),
            "id": "tags",
            "icon": "tags",
            "url": self.context.absolute_url() + "/@@fc-tags",
            "form": {
                "title": translate(_("Tags"), context=self.request),
                "template": self.template(
                    vocabulary_url="%splone.app.vocabularies.Keywords"
                    % (base_vocabulary)
                ),
            },
        }


class TagsActionView(ContentsBaseAction):
    required_obj_permission = "Modify portal content"
    success_msg = _("Successfully updated tags on items")
    failure_msg = _("Failed to modify tags on items")

    def action(self, obj):
        toadd = self.request.form.get("toadd")
        if toadd:
            toadd = set(toadd.split(","))
        else:
            toadd = set()
        toremove = self.request.get("toremove")
        if toremove:
            toremove = set(toremove.split(","))
        else:
            toremove = set()
        tags = set(obj.Subject())
        tags = tags - toremove
        tags = tags | toadd
        obj.setSubject(list(tags))
        obj.reindexObject()



from plone.app.content.testing import PLONE_APP_CONTENT_DX_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.base.utils import is_expired
from zExceptions import Forbidden
from zope.component import getMultiAdapter

import unittest


class TestContentPublishing(unittest.TestCase):
    """Test the recursive behaviour of folder_publish.

    Adapted from CMFPlone/tests/testContentPublishing.py.
    """

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.workflow = self.portal.portal_workflow
        # Make sure we can create and publish directly.
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # Prepare content.
        self.portal.invokeFactory("Folder", id="folder")
        self.folder = self.portal.folder
        self.folder.invokeFactory("Document", id="d1", title="Doc 1")
        self.folder.invokeFactory("Folder", id="f1", title="Folder 1")
        self.folder.f1.invokeFactory("Document", id="d2", title="Doc 2")
        self.folder.f1.invokeFactory("Folder", id="f2", title="Folder 2")

    def setup_authenticator(self):
        from plone.protect.authenticator import createToken

        self.request.form["_authenticator"] = createToken()

    def test_folder_publish_get(self, **kwargs):
        self.setup_authenticator()
        view = getMultiAdapter((self.folder, self.request), name="folder_publish")
        with self.assertRaises(Forbidden):
            view(**kwargs)

    def test_folder_publish_post_without_authenticator(self, **kwargs):
        self.request.environ["REQUEST_METHOD"] = "POST"
        # request.set("REQUEST_METHOD", "POST")
        view = getMultiAdapter((self.folder, self.request), name="folder_publish")
        with self.assertRaises(Forbidden):
            view(**kwargs)

    def folder_publish(self, **kwargs):
        self.request.set("REQUEST_METHOD", "POST")
        self.setup_authenticator()
        view = getMultiAdapter((self.folder, self.request), name="folder_publish")
        return view(**kwargs)

    def test_initial_state(self):
        # Depending on the Plone version,
        # the review state may be visible or private.  Check which one it is.
        for o in (self.folder.d1, self.folder.f1, self.folder.f1.d2, self.folder.f1.f2):
            self.assertEqual(self.workflow.getInfoFor(o, "review_state"), "private")

    def test_publishing_subobjects(self):
        paths = []
        for o in (self.folder.d1, self.folder.f1):
            paths.append("/".join(o.getPhysicalPath()))

        self.folder_publish(
            workflow_action="publish", paths=paths, include_children=True
        )
        for o in (self.folder.d1, self.folder.f1, self.folder.f1.d2, self.folder.f1.f2):
            self.assertEqual(self.workflow.getInfoFor(o, "review_state"), "published")
        self.assertEqual(self.request.response.getStatus(), 302)
        self.assertEqual(
            self.request.response.getHeader("Location"), self.folder.absolute_url()
        )

    def test_publishing_subobjects_and_expire_them(self):
        paths = []
        for o in (self.folder.d1, self.folder.f1):
            paths.append("/".join(o.getPhysicalPath()))

        self.folder_publish(
            workflow_action="publish",
            paths=paths,
            effective_date="1/1/2001",
            expiration_date="1/2/2001",
            include_children=True,
        )
        for o in (self.folder.d1, self.folder.f1, self.folder.f1.d2, self.folder.f1.f2):
            self.assertEqual(self.workflow.getInfoFor(o, "review_state"), "published")
            self.assertTrue(is_expired(o))

    def test_publishing_without_subobjects(self):
        paths = []
        for o in (self.folder.d1, self.folder.f1):
            paths.append("/".join(o.getPhysicalPath()))

        self.folder_publish(
            workflow_action="publish", paths=paths, include_children=False
        )
        for o in (self.folder.d1, self.folder.f1):
            self.assertEqual(self.workflow.getInfoFor(o, "review_state"), "published")
        for o in (self.folder.f1.d2, self.folder.f1.f2):
            self.assertEqual(self.workflow.getInfoFor(o, "review_state"), "private")

    def test_publishing_orig_template_safe(self):
        paths = []
        for o in (self.folder.d1, self.folder.f1):
            paths.append("/".join(o.getPhysicalPath()))

        self.request.form["orig_template"] = "some_view"
        self.folder_publish(workflow_action="publish", paths=paths)
        self.assertEqual(self.request.response.getHeader("Location"), "some_view")

    def test_publishing_orig_template_attacker(self):
        paths = []
        for o in (self.folder.d1, self.folder.f1):
            paths.append("/".join(o.getPhysicalPath()))

        self.request.form["orig_template"] = "https://attacker.com"
        self.folder_publish(workflow_action="publish", paths=paths)
        self.assertEqual(
            self.request.response.getHeader("Location"), self.folder.absolute_url()
        )



from zope import schema
from zope.interface import Attribute
from zope.interface import Interface


class INameFromTitle(Interface):
    """An object that supports gettings it name from its title."""

    title = schema.TextLine(
        title="Title",
        description="A title, which will be converted to a name",
        required=True,
    )


class IReindexOnModify(Interface):
    """Marker interface which makes sure an object gets reindexed when
    it's modified.
    """


class IStructureAction(Interface):
    order = Attribute("Order the action should be listed")

    def get_options():
        """
        Return a dict of action widget options.

        Options: {
            'title': 'Button title', // required
            'name': 'short name', // required
            'formTemplate': None, // If action requires form to submit additional options
            'icon': None, // icon name
            'button-type': 'danger', //
        }
        """



from Acquisition import aq_inner
from plone.app.content.browser.tableview import Table
from plone.app.content.browser.tableview import TableBrowserView
from plone.base.utils import human_readable_size
from plone.base.utils import is_expired
from plone.base.utils import safe_text
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from urllib.parse import quote_plus
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.publisher.browser import BrowserView


class FullReviewListView(BrowserView):
    def revlist(self):
        portal_membership = getToolByName(self.context, "portal_membership")
        portal_workflow = getToolByName(self.context, "portal_workflow")
        if portal_membership.isAnonymousUser():
            return []

        return portal_workflow.getWorklistsResults()

    def url(self):
        return self.context.absolute_url() + "/full_review_list"

    def review_table(self):
        table = ReviewListTable(self.context, self.request)
        return table.render()


class ReviewListTable:
    """
    The review list table renders the table and its actions.
    """

    def __init__(self, context, request, **kwargs):
        self.context = context
        self.request = request

        url = self.context.absolute_url()
        view_url = url + "/full_review_list"
        self.table = Table(request, url, view_url, self.items, buttons=self.buttons)

    def render(self):
        return self.table.render()

    @property
    def items(self):
        portal_url = getToolByName(self.context, "portal_url")
        plone_view = getMultiAdapter((self.context, self.request), name="plone")
        portal_workflow = getToolByName(self.context, "portal_workflow")
        portal_types = getToolByName(self.context, "portal_types")
        portal_membership = getToolByName(self.context, "portal_membership")
        plone_utils = getToolByName(self.context, "plone_utils")

        registry = getUtility(IRegistry)
        use_view_action = registry.get("plone.types_use_view_action_in_listings", ())
        browser_default = plone_utils.browserDefault(self.context)

        results = []
        if portal_membership.isAnonymousUser():
            worklist = []
        else:
            worklist = portal_workflow.getWorklistsResults()

        for i, obj in enumerate(worklist):
            if i % 2 == 0:
                table_row_class = "even"
            else:
                table_row_class = "odd"

            url = obj.absolute_url()
            path = "/".join(obj.getPhysicalPath())
            normalizer = getUtility(IIDNormalizer)
            type_class = "contenttype-" + normalizer.normalize(obj.portal_type)

            review_state = portal_workflow.getInfoFor(obj, "review_state", "")

            state_class = "state-" + normalizer.normalize(review_state)
            relative_url = portal_url.getRelativeContentURL(obj)

            type_title_msgid = portal_types[obj.portal_type].Title()
            url_href_title = "{}: {}".format(
                translate(type_title_msgid, context=self.request),
                safe_text(obj.Description()),
            )
            getMember = getToolByName(obj, "portal_membership").getMemberById
            creator_id = obj.Creator()
            creator = getMember(creator_id)
            if creator:
                creator_name = creator.getProperty("fullname", "") or creator_id
            else:
                creator_name = creator_id
            modified = "".join(
                map(
                    safe_text,
                    [
                        creator_name,
                        " - ",
                        plone_view.toLocalizedTime(
                            obj.ModificationDate(), long_format=1
                        ),
                    ],
                )
            )
            is_structural_folder = obj.restrictedTraverse(
                "@@plone"
            ).isStructuralFolder()

            if obj.portal_type in use_view_action:
                view_url = url + "/view"
            elif is_structural_folder:
                view_url = url + "/folder_contents"
            else:
                view_url = url

            is_browser_default = len(browser_default[1]) == 1 and (
                obj.id == browser_default[1][0]
            )

            results.append(
                dict(
                    url=url,
                    url_href_title=url_href_title,
                    id=obj.getId(),
                    quoted_id=quote_plus(obj.getId()),
                    path=path,
                    title_or_id=obj.pretty_title_or_id(),
                    description=obj.Description(),
                    obj_type=obj.Type,
                    size=human_readable_size(obj.get_size()),
                    modified=modified,
                    type_class=type_class,
                    wf_state=review_state,
                    state_title=portal_workflow.getTitleForStateOnType(
                        review_state, obj.portal_type
                    ),
                    state_class=state_class,
                    is_browser_default=is_browser_default,
                    folderish=is_structural_folder,
                    relative_url=relative_url,
                    view_url=view_url,
                    table_row_class=table_row_class,
                    is_expired=is_expired(obj),
                )
            )
        return results

    @property
    def show_sort_column(self):
        return False

    def buttons(self):
        buttons = []
        portal_actions = getToolByName(self.context, "portal_actions")
        button_actions = portal_actions.listActionInfos(
            object=aq_inner(self.context), categories=("folder_buttons",)
        )

        for button in button_actions:
            # Make proper classes for our buttons
            if button["id"] != "paste" or self.context.cb_dataValid():
                buttons.append(self.setbuttonclass(button))
        return buttons

    def setbuttonclass(self, button):
        if button["id"] == "paste":
            button["cssclass"] = "btn btn-secondary"
        else:
            button["cssclass"] = "btn btn-primary"
        return button


class ReviewListBrowserView(TableBrowserView):
    table = ReviewListTable



from plone.app.content.namechooser import ATTEMPTS
from plone.app.content.testing import PLONE_APP_CONTENT_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from zope.container.interfaces import INameChooser

import transaction
import unittest


class NameChooserTest(unittest.TestCase):

    layer = PLONE_APP_CONTENT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        portal_types = getToolByName(self.portal, "portal_types")
        if "Document" not in portal_types.objectIds():
            fti = DexterityFTI("Document")
            portal_types._setObject("Document", fti)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)

    def test_100_or_more_unique_ids(self):
        # add the same item 110 times. the first 100 items should be numbered.
        # after that it should use datetime to generate the id

        holder = self.portal
        title = "A Small Document"
        # create the first object, which will have no suffix
        holder.invokeFactory("Document", id="a-small-document")

        chooser = INameChooser(holder)

        for i in range(1, ATTEMPTS + 1):
            id = chooser.chooseName(title, holder)
            if i <= ATTEMPTS:  # first addition has no suffix
                self.assertEqual("a-small-document-%s" % i, id)
            else:
                self.assertNotEqual("a-small-document-%s" % i, id)

            holder.invokeFactory("Document", id)
            transaction.savepoint(optimistic=True)
            holder.get(id)



from plone.base import PloneMessageFactory as _
from plone.base.utils import human_readable_size
from plone.base.utils import is_expired
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import field
from z3c.form import form
from zope.deprecation.deprecation import deprecate
from zope.interface import Interface
from zope.publisher.browser import BrowserView
from zope.schema import Datetime
from zope.schema.fieldproperty import FieldProperty


class IContentStatusHistoryDates(Interface):
    """Interface for the two dates on content status history view"""

    effective_date = Datetime(
        title=_("label_effective_date", default="Publishing Date"),
        description=_(
            "help_effective_date_content_status_history",
            default="The date when the item will be published. If no "
            "date is selected the item will be published immediately.",
        ),
        required=False,
    )

    expiration_date = Datetime(
        title=_("label_expiration_date", default="Expiration Date"),
        description=_(
            "help_expiration_date_content_status_history",
            default="The date when the item expires. This will automatically "
            "make the item invisible for others at the given date. "
            "If no date is chosen, it will never expire.",
        ),
        required=False,
    )


class ContentStatusHistoryDatesForm(form.Form):
    fields = field.Fields(IContentStatusHistoryDates)
    ignoreContext = True
    label = "Content status history dates"

    effective_date = FieldProperty(IContentStatusHistoryDates["effective_date"])
    expiration_date = FieldProperty(IContentStatusHistoryDates["expiration_date"])


class ContentStatusHistoryView(BrowserView):

    template = ViewPageTemplateFile("templates/content_status_history.pt")

    def __init__(self, context, request):
        super().__init__(context, request)

        self.dates_form = ContentStatusHistoryDatesForm(context, request)
        self.dates_form.updateWidgets()
        self.errors = {}

    def __call__(
        self,
        workflow_action=None,
        paths=[],
        comment="",
        effective_date=None,
        expiration_date=None,
        include_children=False,
        *args
    ):

        data = self.dates_form.extractData()
        if self.request.get("form.widgets.effective_date-calendar", None) and data:
            effective_date = data[0]["effective_date"].strftime("%Y-%m-%d %H:%M")

        if self.request.get("form.widgets.expiration_date-calendar", None) and data:
            expiration_date = data[0]["expiration_date"].strftime("%Y-%m-%d %H:%M")

        if self.request.get("form.button.Cancel", None):
            return self.request.RESPONSE.redirect(
                "%s/view" % self.context.absolute_url()
            )

        if self.request.get("form.submitted", None):
            self.validate(workflow_action=workflow_action, paths=paths)
            if self.errors:
                IStatusMessage(self.request).add(
                    _("Please correct the indicated errors."), type="error"
                )
                return self.template()

        if self.request.get("form.button.Publish", None):
            return self.context.restrictedTraverse("content_status_modify")(
                workflow_action=workflow_action,
                comment=comment,
                effective_date=effective_date,
                expiration_date=expiration_date,
            )

        if self.request.get("form.button.FolderPublish", None):
            self.context.restrictedTraverse("folder_publish")(
                workflow_action=workflow_action,
                paths=paths,
                comment=comment,
                expiration_date=expiration_date,
                effective_date=effective_date,
                include_children=include_children,
            )

        return self.template()

    def validate(self, workflow_action=None, paths=[]):
        if workflow_action is None:
            self.errors["workflow_action"] = _("You must select a publishing action.")

        if not paths:
            self.errors["paths"] = _("You must select content to change.")
            # If there are no paths, it's mostly a mistake
            # Set paths using orgi_paths, otherwise users are getting confused
            orig_paths = self.request.get("orig_paths")
            self.request.set("paths", orig_paths)

    def get_objects_from_path_list(self, paths=[]):
        contents = []
        portal = getToolByName(self.context, "portal_url").getPortalObject()
        for path in paths:
            obj = portal.restrictedTraverse(str(path), None)
            if obj is not None:
                contents.append(obj)
        return contents

    def redirect_to_referrer(self):
        referer = self.request.get("HTTP_REFERER", "")
        target_url = referer.split("?", 1)[0]
        return self.request.RESPONSE.redirect(target_url)

    def isExpired(self, content):
        return is_expired(content)

    @deprecate(
        "This method is deprecated since Plone 6, "
        "use the @@plone/human_readable_size method instead"
    )
    def human_readable_size(self, size):
        return human_readable_size(size)



from AccessControl import Unauthorized
from Acquisition import aq_inner
from Acquisition import aq_parent
from DateTime import DateTime
from plone.base import PloneMessageFactory as _
from plone.base.defaultpage import check_default_page_via_view
from plone.base.utils import transaction_note
from plone.protect import CheckAuthenticator
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter


class ContentStatusModifyView(BrowserView):
    """Handles the workflow transitions of objects.

    Former Controller Python Script "content_status_modify".

    [validators]
    validators = validate_content_status_modify

    [actions]
    action.failure=traverse_to:string:content_status_history
    action.success=redirect_to_action:string:view

    """

    def __call__(
        self,
        workflow_action=None,
        comment="",
        effective_date=None,
        expiration_date=None,
        **kwargs,
    ):
        """Do a workflow action.

        The status dropdown in the toolbar has links to for example:
        content_status_modify?workflow_action=reject&_authenticator=secret
        That is the main entry into this browser view.

        When you right-click the status menu and open in a new tab,
        you end up on the content_status_history page.
        This page contains a form which posts to this view.

        In the form you can select a workflow action.
        When you select "no change" the workflow_action parameter actually contains
        the current status.  This status is naturally not in the list of allowed transitions.
        So we should be lenient here, and not complain much.

        Also, when the view is called on a default page,
        the code below tries to do the same transition on the parent folder,
        by calling this view on the parent.  This may easily fail.
        This is yet another reason to be lenient.
        Otherwise you may see both a successful portal status message
        and one with an error.

        In the form you can also add a comment and set an effective and/or expiration date.
        """
        context = aq_inner(self.context)
        portal_workflow = getToolByName(context, "portal_workflow")
        self.plone_utils = getToolByName(context, "plone_utils")
        # First check if the main argument is given.
        if not workflow_action:
            IStatusMessage(self.request).add(
                _("You must select a publishing action."), type="error"
            )
            url = f"{context.absolute_url()}/content_status_history"
            return self.request.response.redirect(url)
        # If a workflow action was specified, there must be a plone.protect authenticator.
        CheckAuthenticator(self.request)

        # Get effective and expiration dates from the form, if not set in the arguments.
        form = self.request.form
        if not effective_date:
            effective_date = form.get("form.widgets.effective_date") or effective_date
        if not expiration_date:
            expiration_date = (
                form.get("form.widgets.expiration_date") or expiration_date
            )

        # Make sure an effective date is always set when there is a valid workflow action.
        transitions = portal_workflow.getTransitionsFor(context)
        transition_ids = [t["id"] for t in transitions]
        if (
            workflow_action in transition_ids
            and not effective_date
            and context.EffectiveDate() == "None"
        ):
            effective_date = DateTime()

        # You can transition content but not have the permission to ModifyPortalContent.
        contentEditSuccess = 0
        try:
            self.editContent(context, effective_date, expiration_date)
            contentEditSuccess = 1
        except Unauthorized:
            pass

        # Create the note while we still have access to the original context
        note = "Changed status of {} at {}".format(
            context.title_or_id(),
            context.absolute_url(),
        )

        if workflow_action in transition_ids:
            # The action could result in a move or delete.
            # In that case we get a new context as answer.
            context = portal_workflow.doActionFor(
                context, workflow_action, comment=comment
            )
            if context is None:
                # the normal case
                context = aq_inner(self.context)

        # The object post-transition could now have ModifyPortalContent permission.
        if not contentEditSuccess:
            try:
                self.editContent(context, effective_date, expiration_date)
            except Unauthorized:
                pass

        transaction_note(note)

        # If this item is the default page in its parent, attempt to publish that
        # too. It may not be possible, of course
        if check_default_page_via_view(context, self.request):
            parent = aq_parent(context)
            try:
                parent_modify_view = getMultiAdapter(
                    (parent, self.request), name="content_status_modify"
                )
                parent_modify_view(
                    workflow_action,
                    comment,
                    effective_date=effective_date,
                    expiration_date=expiration_date,
                )
            except ConflictError:
                raise
            except Exception:
                pass

        IStatusMessage(self.request).add(_("Item state changed."))
        return self.request.response.redirect(context.absolute_url())

    def editContent(self, obj, effective, expiry):
        kwargs = {}
        # may contain the year
        if effective and (isinstance(effective, DateTime) or len(effective) > 5):
            kwargs["effective_date"] = effective
        # may contain the year
        if expiry and (isinstance(expiry, DateTime) or len(expiry) > 5):
            kwargs["expiration_date"] = expiry
        self.plone_utils.contentEdit(obj, **kwargs)



from Acquisition import Implicit
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.adding import ContentAdding


class CMFAdding(Implicit, ContentAdding):
    """An adding view with a less silly next-url"""

    # We need to do this to get proper traversal URLs - otherwise, the
    # <base /> tag is messed up.
    id = "+"

    def add(self, content):
        content = super().add(content)
        # We need to ensure that we finish type construction, not at least
        # to set the correct permissions based on the workflow
        getToolByName(content, "portal_types")

        return content

    def nextURL(self):
        return f"{self.context.absolute_url()}/{self.contentName}/view"



from DateTime import DateTime
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from plone.app.dexterity.behaviors.metadata import ICategorization
from plone.app.widgets.utils import get_datetime_options
from plone.base import PloneMessageFactory as _
from plone.base.defaultpage import check_default_page_via_view
from plone.dexterity.interfaces import IDexterityContent
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory

import json


@implementer(IStructureAction)
class PropertiesAction:

    template = ViewPageTemplateFile("templates/properties.pt")
    order = 8

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        base_vocabulary = "%s/@@getVocabulary?name=" % getSite().absolute_url()
        return {
            "tooltip": translate(_("Properties"), context=self.request),
            "id": "properties",
            "icon": "plone-edit",
            "url": self.context.absolute_url() + "/@@fc-properties",
            "form": {
                "title": translate(
                    _("Modify properties on items"), context=self.request
                ),
                "template": self.template(
                    vocabulary_url="%splone.app.vocabularies.Users" % (base_vocabulary),
                    pattern_options=json.dumps(get_datetime_options(self.request)),
                ),
                "dataUrl": self.context.absolute_url() + "/@@fc-properties",
            },
        }


class PropertiesActionView(ContentsBaseAction):
    success_msg = _("Successfully updated metadata")
    failure_msg = _("Failure updating metadata")
    required_obj_permission = "Modify portal content"

    def __call__(self):

        if self.request.form.get("render") == "yes":
            lang_factory = getUtility(
                IVocabularyFactory, "plone.app.vocabularies.SupportedContentLanguages"
            )
            lang_vocabulary = lang_factory(self.context)
            languages = [
                {"title": term.title, "value": term.value} for term in lang_vocabulary
            ]
            return self.json(
                {
                    "languages": [
                        {
                            "title": translate(
                                _("label_no_change", default="No change"),
                                context=self.request,
                            ),
                            "value": "",
                        }
                    ]
                    + languages
                }
            )

        self.effectiveDate = self.request.form.get("effectiveDate")
        self.expirationDate = self.request.form.get("expirationDate")
        self.copyright = self.request.form.get("copyright")
        self.contributors = self.request.form.get("contributors")
        if self.contributors:
            self.contributors = self.contributors.split(",")
        else:
            self.contributors = []
        self.creators = self.request.form.get("creators", "")
        if self.creators:
            self.creators = self.creators.split(",")
        self.exclude = self.request.form.get("exclude-from-nav")
        self.language = self.request.form.get("language")
        self.recurse = self.request.form.get("recurse", "no") == "yes"
        return super().__call__()

    def action(self, obj, bypass_recurse=False):

        if check_default_page_via_view(obj, self.request):
            self.action(obj.aq_parent, bypass_recurse=True)
        recurse = self.recurse and not bypass_recurse
        if recurse and IFolderish.providedBy(obj):
            for sub in obj.values():
                self.action(sub)

        if self.effectiveDate and hasattr(obj, "effective_date"):
            obj.effective_date = DateTime(self.effectiveDate)
        if self.expirationDate and hasattr(obj, "expiration_date"):
            obj.expiration_date = DateTime(self.expirationDate)
        if self.copyright and hasattr(obj, "rights"):
            obj.rights = self.copyright
        if self.contributors and hasattr(obj, "contributors"):
            obj.contributors = tuple(self.contributors)
        if self.creators and hasattr(obj, "creators"):
            obj.creators = tuple(self.creators)
        if self.exclude and hasattr(obj, "exclude_from_nav"):
            obj.exclude_from_nav = self.exclude == "yes"

        behavior_categorization = ICategorization(obj)
        if self.language and behavior_categorization:
            behavior_categorization.language = self.language

        obj.reindexObject()



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.content.browser.contents import ContentsBaseAction
from plone.app.content.interfaces import IStructureAction
from plone.base import PloneMessageFactory as _
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.i18n import translate
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent

import logging
import transaction


logger = logging.getLogger("plone.app.content")


@implementer(IStructureAction)
class RenameAction:

    template = ViewPageTemplateFile("templates/rename.pt")
    order = 5

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_options(self):
        return {
            "tooltip": translate(_("Rename"), context=self.request),
            "id": "rename",
            "icon": "plone-rename",
            "url": self.context.absolute_url() + "/@@fc-rename",
            "form": {
                "title": translate(_("Rename"), context=self.request),
                "template": self.template(),
            },
        }


class RenameActionView(ContentsBaseAction):
    success_msg = _("Items renamed")
    failure_msg = _("Failed to rename all items")

    def __call__(self):
        self.errors = []
        self.protect()
        context = aq_inner(self.context)

        catalog = getToolByName(context, "portal_catalog")
        mtool = getToolByName(context, "portal_membership")

        missing = []
        for key in self.request.form.keys():
            if not key.startswith("UID_"):
                continue
            index = key.split("_")[-1]
            uid = self.request.form[key]
            brains = catalog(UID=uid, show_inactive=True)
            if len(brains) == 0:
                missing.append(uid)
                continue
            obj = brains[0].getObject()
            title = self.objectTitle(obj)
            if not mtool.checkPermission("Copy or Move", obj):
                self.errors.append(
                    _("Permission denied to rename ${title}.", mapping={"title": title})
                )
                continue

            sp = transaction.savepoint(optimistic=True)

            newid = self.request.form["newid_" + index]
            newtitle = self.request.form["newtitle_" + index]
            try:
                obid = obj.getId()
                title = obj.Title()
                change_title = newtitle and title != newtitle
                if change_title:
                    getSecurityManager().validate(obj, obj, "setTitle", obj.setTitle)
                    obj.setTitle(newtitle)
                    notify(ObjectModifiedEvent(obj))
                if newid and obid != newid:
                    parent = aq_parent(aq_inner(obj))
                    # Make sure newid is safe
                    newid = INameChooser(parent).chooseName(newid, obj)
                    # Update the default_page on the parent.
                    context_state = getMultiAdapter(
                        (obj, self.request), name="plone_context_state"
                    )
                    if context_state.is_default_page():
                        parent.setDefaultPage(newid)
                    parent.manage_renameObjects((obid,), (newid,))
                elif change_title:
                    # the rename will have already triggered a reindex
                    obj.reindexObject()
            except ConflictError:
                raise
            except Exception as e:
                sp.rollback()
                logger.error(
                    'Error renaming "{title}": "{exception}"'.format(
                        title=title, exception=e
                    )
                )
                self.errors.append(
                    _("Error renaming ${title}", mapping={"title": title})
                )

        return self.message(missing)
